//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>       
#include "PSoCAPI.h"    
#include "stdlib.h"		
//#include "stdint.h"
#include "delay.h"

///////////////******************************************NEU
//Diese Daten-Typen sind stdint.h Bibliothek enthalten
//siehe PDF-Datei: "ImageCraft C Compiler Guide" auf der Internetseite: http://www.cypress.com/file/44751/download
typedef signed char int8_t;
typedef unsigned char uint8_t;
typedef short int16_t;
typedef unsigned short uint16_t;
typedef long int32_t;
typedef unsigned long uint32_t;
typedef long long int64_t;
typedef unsigned long long uint64_t;
///////////////******************************************

	//Funktion für die Ausgabe auf UART (Float als String ausgeben)
	void FloatToString (float Var);
	
	//Funktion für die Ausagbe auf UART (Integer als String ausgeben)
	void IntToString (int16_t Var);

	//Funktion um das AD-Modul zu konfiguieren, s. unten
	void DUALADC_Initi(void);

	//Funktion um das UART-Modul zu konfiguieren, s. unten
	void UART_Init(void);

	//damit beim Einschalten (1st Durchlauf) Werte berechnet werden können
	void U_GAS_vorbereitung(void);
	
//	int string_to_int (char *Var);
	
	//Faktor um den Analogen Wert des Gashebels an Controller auszugeben, s. Funktion DAC8_WriteBlind
	//Berechnung des AD-Wandler Faktors um Analogen Wert mit Funktion DAC8_WriteBlind
	//4,75V = max. Spannung des Gaszuggriffes 
	//256= 8 bit Auflösung
	float fScaleFactor_analog = (float)4.8/(float)255;

	//Faktor für die Umrechnung von int zu float, s. im Hauptprogramm bei U_Gas_Neu
	//Berechnung des AD-Wandler Faktors um Gleitkommazahlen ausgegeben zu bekommen
	//4,75V = max. Spannung des Gaszuggriffes
	//256= 8 bit Auflösung
	float fScaleFactorSpannung = (float)4.8/(float)256;
	
	//Faktor für die Umrechnung von int zu float, s. im Hauptprogramm bei I_Hall_Neu
	//Berechnung des AD-Wandler Faktors um Gleitkommazahlen ausgegeben zu bekommen
	//63A = max. Strom des Hallsensors
	//256= 8 bit Auflösung
	float fScaleFactorStrom = (float)100/(float)256;

	//Werte des AD_Wandlers 2 werden als Integer abgespeichert
	int U_Gas; 

	//Werte des AD_Wandlers 1 werden als Integer abgespeichert
	int I_Hall;

	//Variable float (Kommazahl) für U_Gas_Neu
	float U_Gas_Neu;
	
	//Variable float (Kommazahl) für U_Gas_alt
	float U_Gas_alt;

	//Variable float (Kommazahl) für I_Hall_Neu
	float I_Hall_Neu;

	
	
void main(void)
{
	char *strPtr;
	int Pitch, Roll;
	
	//Globale Interrupts werden aufgerufen
	M8C_EnableGInt;	
	
	//Das AD-Modul wird durch diese Funktion konfoguriert, s.Funktion unten
	DUALADC_Initi();
	
	//PGA_1_Start startet den PGA_1
	PGA_1_Start(PGA_1_HIGHPOWER);
	
	//PGA_2_Start startet den PGA_2
	PGA_2_Start(PGA_2_HIGHPOWER);
	
	//Das UART-Modul wird durch diese Funktion konfoguriert, s.Funktion unten
	UART_Init();
			
	//Der DA-Wandler wird für die Ausgebe der Spannung  
	//des Gashebels an den Controller mit "FULLPOWER" gestartet
	DAC8_Start(DAC8_HIGHPOWER);
	
	///
	U_GAS_vorbereitung();	
	//
	while (1)
	{
		//If-Schleife um das Abtasten des DUALADCs zu beginnen
		if(DUALADC_fIsDataAvailable() ==0);
		{
			//Abgetasteter Integer-Wert des AD-Wandlers 2 wird in U_Gas abgespeichert
			I_Hall = DUALADC_iGetData2();
			
			//Abgetasteter Integer-Wert des AD-Wandlers 1 wird in I_Hall abgespeichert
			U_Gas  = DUALADC_iGetData1();
			
			//Mit diesen Befehl wird der Inhalt im AD-Wandler Eingang 1+2 nach abspeichern
			//von U_Gas und I_Hall gelöscht
			DUALADC_ClearFlag();
		}

		
		//Die float Variable U_Gas_Neu wird wie folgt berechnet, damit eine Kommazahl ausgegeben werden kann
		U_Gas_Neu = (float)U_Gas * fScaleFactorSpannung;

		UART_CPutString("\n\r");UART_CPutString("U-Gas-Eingelesen->");FloatToString(U_Gas_Neu);UART_CPutString("\t"); 
		
		//Die float Variable U_Gas_Neu wird wie folgt berechnet, damit eine Kommazahl ausgegeben werden kann
		I_Hall_Neu = (float)I_Hall * fScaleFactorStrom; 
				
		UART_CPutString("I-Hall-Eingelesen->");FloatToString(I_Hall_Neu); UART_CPutString("\n\r");
					
        //MPU BUFFER
		if (UART_bCmdCheck()) 
		{   
			if  (strPtr = UART_szGetParam()) 
			{ 
				Pitch = atoi(strPtr);
			}
			if  (strPtr = UART_szGetParam()) 
			{ 
				Roll = atoi(strPtr);
			}
		UART_CmdReset();
		}
		
		//Mit der If-Schleife wird der float Wert von U_Gas_Neu überprüft, 
		//ob U_Gas_Neu größer als 2.7(V) ist (max. 1600W), wenn ja dann springt es in die If-Schleife
		if (U_Gas_Neu > 2.7)
		{
			//Dann soll der float Wert U_Gas_Neu max. 2.7(V) betragen
			U_Gas_Neu = 2.7;
		}	
					
		//STROMBEGRENZUNG
		//Mit der If-Schleife wird der float Wert von I_Hall_Neu überprüft, 
		//ob I_Hall_Neu größer als 33.3(V) ist (max. 1600W), wenn ja dann springt es in die If-Schleife
		if (I_Hall_Neu > 33.3)
		{
			U_Gas_Neu = U_Gas_alt - 0.01;
				
			//Mit der If-Schleife wird der float Wert von I_Hall_Neu überprüft, 
			//ob I_Hall_Neu größer als Pitch ist, wenn ja dann springt es in die If-Schleife
			if (Pitch > 5)
			{	//Dann soll der float Wert U_Gas_Neu max. 2.7(V) betragen
					U_Gas_Neu = U_Gas_Neu - 0.01;
			}
		}	
        
		U_Gas_alt = U_Gas_Neu;
				
		//Der DA-Wandler gibt den ausgerechneten positiven float Wert (DatenFormat=OffsetBinary)
        //von 0 bis 256, welcher am Gashebel gedreht wurde, aus
		DAC8_WriteBlind(U_Gas_Neu / fScaleFactor_analog);
		
	
//		UART AUSGABE 
		UART_CPutString("Steigung->");IntToString(Pitch); UART_CPutString("\t"); 
		UART_CPutString("Neigung->");IntToString(Roll); UART_CPutString("\n\r");
		
		//UART AUSGABE Überprüfung für U-Gas-Aus
		UART_CPutString("U-Gas-Ausgabe>");FloatToString(U_Gas_Neu);UART_CPutString("\n\r");
//		Delay10msTimes(255);
	}
}

	//Hier wird der DUALADC Initialisiert
	void DUALADC_Initi(void)
{
	 //DUALADC_Start startet den AD-Wandler
	DUALADC_Start (DUALADC_HIGHPOWER);
	
	//Durch diesen Befehl wird die AD-Wandler-Abtastung der Werte gestartet
	DUALADC_GetSamples (0);
}


	//Mit dieser Funktion wird das UART initialisiert
	void UART_Init(void)
{
	/*Initialisierung vom RX und das Cmd Buffers*/ 
    UART_CmdReset(); 
	
	//Aktiviert und wählt beide UART-Interrupts (TX und RX), durch setzten des benötigten
	//Interrupt bits
	UART_EnableInt();
	
	//UART-Schnittstelle wird gestartet
	UART_Start(UART_PARITY_NONE);
}

	void IntToString (int16_t Var)
{	
	char Array[17]; // Notiz zur Berechnung der Array Größe = char buffer [sizeof(int)*8+1]; 
	char* pVar;
	pVar=itoa(Array,Var,10); // stdlib
	UART_PutString(pVar);
}

	void FloatToString (float Var)
{	
	int  status;
	char * pVar;
	pVar=ftoa(Var,&status);
	UART_PutString(pVar);
}

	void U_GAS_vorbereitung(void)
{
	//If-Schleife um das Abtasten des DUALADCs zu beginnen
	if(DUALADC_fIsDataAvailable() ==0);
	{
		//Abgetasteter Integer-Wert des AD-Wandlers 2 wird in U_Gas abgespeichert
		I_Hall = DUALADC_iGetData2();
		
		//Abgetasteter Integer-Wert des AD-Wandlers 1 wird in I_Hall abgespeichert
		U_Gas  = DUALADC_iGetData1();
		
		//Mit diesen Befehl wird der Inhalt im AD-Wandler Eingang 1+2 nach abspeichern
		//von U_Gas und I_Hall gelöscht
		DUALADC_ClearFlag();
	}
	//Die float Variable U_Gas_Neu wird wie folgt berechnet, damit eine Kommazahl ausgegeben werden kann
	U_Gas_alt = (float)U_Gas * fScaleFactorSpannung;
}

//	int string_to_int (char *Var)
//{
//	return atoi (Var);
//}